/*
  new_mathematics.cpp implements replacements with much better performance for some trigonometric functions
  sin,  cos,  tan, atan2  and sqrt 
  can now be replaced with
  fsin, fcos, ftan, fatan2 and fsqrt.

  Thanks to Oscar Liang for his work on this (source: http://blog.oscarliang.net/enhanced-arduino-c-custom-math-library/).
  Integration into Marlin, transformation to RAD-parameters and math_tables by Robert Kuhlmann.

  June, 2014, RobertKuhlmann, Germany
*/

/*
  Useful stuff

  radians = (degrees * 71) / 4068

  degrees = (radians * 4068) / 71

  With floating point this is more accurate than using the "original" values
*/

float fsin(float rad)
{
	int deg = int((rad * 40680) / 71);
	return SIN_TABLE[deg];
} //float fsin(float rad)

float fcos(float rad)
{
	int deg = int((rad * 40680) /71);
	return COS_TABLE[deg];
} //float fcos(float rad)

float ftan(float rad)
{
	int deg = int((rad * 40680) /71);
	return TAN_TABLE[deg];
}//float ftan(float rad)

float facos(float num)
{
	return ACOS_TABLE[2048*(num+1)];
}//float facos(float pipart)

float fatan2(float opp, float adj)
{

	float hypt = fsqrt(adj * adj + opp * opp);
	float rad = facos(adj/hypt);

	if(opp < 0)
		rad = -rad;

	return rad;
}//float fatan2(float opp, float adj)

float sqrt(float num)
{
	return sqrt(num); //better way is under construction
	/*
	The following code for ATmega8 was written by someone at the university of Chemnitz, Germany, back in 2005.
	It needs to be checked for modifications necessary for the new ATmega256, but maybe everything's okay already.
	Don't know.

.def RL = r2	;Radikand (24 bit), Datenquelle
.def RM = r3
.def RH = r4
.def AL = r12	;Akku (14 bit), Rest (13bit) am Ende
.def AH = r13
.def WL = r18	;Ungerade Doppelwurzel (13bit), Wurzel (12bit) am Ende
.def WH = r19
.def LoopCounter = r17	;Schleifenzhler

;ca. 280 Takte (17,5 s @ 16 MHz)
Wurzel:
;Doppelwurzel = 0 (LSB wird sowieso gesetzt)
	clr	WL
	clr	WH
;Akku lschen
	clr	AL
	clr	AH

	ldi	LoopCounter,12	;Fr jedes Ergebnis-Bit eine Runde
zieh:
;Doppelwurzel "verdoppeln"
	sec
	rol	WL	;[xxxxx1 -> xxxxx01]
	rol	WH
	cbr	WL,2
;Zwei Bits aus Radikand in Akku schieben
	lsl	RL
	rol	RM
	rol	RH
	rol	AL
	rol	AH
	lsl	RL
	rol	RM
	rol	RH
	rol	AL
	rol	AH
;	brcs	setbit	;auskommentieren fr 22/30-bit-Radikand
;Akku mit Doppelwurzel vergleichen
	cp	AL,WL
	cpc	AH,WH
	brcs	okay
setbit:
;Doppelwurzel vom Akku abziehen
	sub	AL,WL
	sbc	AH,WH
;Doppelwurzel+=2
	sbr	WL,2	;[xxxxx01 -> xxxxx11]
okay:
;nchste Runde
	dec	LoopCounter
	brne	zieh
;Doppelwurzel halbieren
	lsr	WH
	ror	WL
;fertig
	ret

	*/
}//float sqrt(float num)
